/*
    This file is part of Escota.

    Escota is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Escota is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Escota.  If not, see <http://www.gnu.org/licenses/>.
 */

package escota.flickr;

import escota.utils.Log;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.*;

import javax.xml.parsers.ParserConfigurationException;
import org.xml.sax.SAXException;

import com.aetrion.flickr.Flickr;
import com.aetrion.flickr.FlickrException;
import com.aetrion.flickr.REST;
import com.aetrion.flickr.RequestContext;
import com.aetrion.flickr.auth.Auth;
import com.aetrion.flickr.auth.Permission;
import com.aetrion.flickr.util.AuthStore;
import com.aetrion.flickr.util.FileAuthStore;
import com.aetrion.flickr.groups.*;
import com.aetrion.flickr.photosets.Photoset;
import com.aetrion.flickr.photosets.PhotosetsInterface;
import com.aetrion.flickr.photos.PhotoList;
import com.aetrion.flickr.photos.Photo;
import com.aetrion.flickr.photos.PhotosInterface;
import com.aetrion.flickr.photos.Size;
import com.aetrion.flickr.tags.Tag;
import escota.EscotaView;
import java.io.BufferedInputStream;
import java.io.FileOutputStream;
import javax.swing.ImageIcon;
import javax.swing.JProgressBar;
/**
 *
 * @author AGuS
 */
public class Conexion {

    private String nsid = null;
    private Flickr flickr = null;
    private AuthStore authStore = null;
    private String secret = null;
    private String apiKey = null;
    private Boolean authorized = false;
    private String frob = "";
    private static String directory = System.getProperty("user.home") + File.separatorChar + ".escota";
    private static File authsDir =  new File(directory  + File.separatorChar + ".flickrAuth");
    private Collection myGroups = null;
    private Map allPhotos = null;
    private boolean dataUpdated = false;
    private EscotaView escotaView = null;
    private boolean step2 = false;
    private int numberPhotos = 0;
    private boolean canAuth = false;

    public boolean isStep2() {
        return step2;
    }

    public EscotaView getEscotaView() {
        return escotaView;
    }
    
    public boolean canAuth()
    {
        return canAuth;
    }
    public void setCanAuth(boolean canAuth)
    {
        this.canAuth = canAuth;
    }

    public void setStep2(boolean step2) {
        this.step2 = step2;
    }

    public boolean isDataUpdated() {
        return dataUpdated;
    }
    public String getSecret()
    {
        return secret;
    }

    public AuthStore getAuthStore() {
        return authStore;
    }

    public Flickr getFlickr() {
        return flickr;
    }

    public String getNsid() {
        return nsid;
    }
    public static List<String> retriveAllUserStored() throws IOException
    {
        List<String> nsidList = new ArrayList<String>();
        if(authsDir.exists())
        {
            AuthStore au = new FileAuthStore(authsDir);
            Auth[] array = au.retrieveAll();
            for(int i=0; i<array.length;i++)
            {
                nsidList.add(array[i].getUser().getUsername());
            }
        }
        return nsidList;
    }
    public static String getNSID(String userName) throws IOException
    {
        String nsid = "";
        AuthStore au = new FileAuthStore(authsDir);
        Auth[] array = au.retrieveAll();

        for(int i=0; i<array.length;i++)
        {
            if(array[i].getUser().getUsername().equalsIgnoreCase(userName))
               return array[i].getUser().getId();

        }
        return nsid;
    }
    public String getGroupID(String groupName)
    {
        String res = "";
        Iterator<Group> it = myGroups.iterator();
        while(it.hasNext())
        {
            Group g = it.next();
            if(groupName.equalsIgnoreCase(g.getName()))
                res = g.getId();
        }
        return res;
    }
    public String getSetID(String setName)
    {
        String res = "";
        Iterator<Photoset> it = allPhotos.keySet().iterator();
        while(it.hasNext())
        {
            Photoset s = it.next();
            if(setName.equalsIgnoreCase(s.getId()))
                res = s.getId();
        }
        return res;
    }
    public String getGroupName(String groupID)
    {
        String res = "";
        Iterator<Group> it = myGroups.iterator();
        while(it.hasNext())
        {
            Group g = it.next();
            if(groupID.equalsIgnoreCase(g.getName()))
                res = g.getName();
        }
        return res;
    }
    public Conexion(EscotaView escotaView)
            throws IOException, ParserConfigurationException
    {
        File direc = new File(directory);
        if (!direc.exists())
            direc.mkdir();


        this.escotaView = escotaView;
        this.apiKey = "058d28d33e2b1cd4b80a9d326fb1b2c9";
        this.secret = "21154ea6d830be68";
        this.flickr = new Flickr(apiKey, secret, new REST());
        this.authStore = new FileAuthStore(authsDir);
        this.authorized = false;
    }
    public void authorize(String nsid) throws IOException, SAXException, FlickrException
    {
        this.nsid = nsid;
        RequestContext rc = RequestContext.getRequestContext();
        Auth auth = this.authStore.retrieve(this.nsid);
        if (auth != null) {
            rc.setAuth(auth);
            this.authorized = true;
        }
    }
    public String authorizeStep1() throws IOException, SAXException, FlickrException {

        frob = this.flickr.getAuthInterface().getFrob();

        URL authUrl = this.flickr.getAuthInterface().buildAuthenticationUrl(Permission.WRITE, frob);
        return authUrl.toExternalForm();

    }
    public void authorizeStep2() throws IOException, SAXException, FlickrException {

        Auth token = this.flickr.getAuthInterface().getToken(frob);
        RequestContext.getRequestContext().setAuth(token);
        if(authStore==null)
            Log.debug("Crear authStore");
        this.authStore.store(token);
        frob="";
        this.nsid = token.getUser().getId();
        this.authorized=true;
    }
    public void updateUserData() throws IOException, SAXException, FlickrException, InterruptedException
    {
        while(escotaView.getUpdatingData()==null)
            Thread.sleep(1000);
        
        escotaView.getUpdatingData().getMessage().setText("Retriving groups that you are subscribed");
        retrieveMyGroups();
        escotaView.getUpdatingData().getMessage().setText("Retriving info about your account");
        allMyPhotosMap();
        escotaView.getUpdatingData().getMessage().setText("Downloading your photos");
        downloadPhotos();
        this.dataUpdated = true;
        escotaView.getUpdatingData().close();
        Log.debug("done");
    }


    /**
     *
     * @param photo
     * @return Returns the number of favorites of the photo
     * @throws java.io.IOException
     * @throws org.xml.sax.SAXException
     * @throws com.aetrion.flickr.FlickrException
     */
    public int numberFavorites(Photo photo) throws IOException, SAXException, FlickrException
    {
        PhotosInterface photoInt = flickr.getPhotosInterface();
        int nfavs = 0, page = 1, last = 0;
        boolean flag = true;

        while (flag)
        {
            last = photoInt.getFavorites(photo.getId(), 50, page).size();
  
            nfavs = nfavs + last;
            if(last==0) flag = false;
            page++;
        }

        return nfavs;
    }

    /**
     *
     * @param groupID The ID of a group
     * @return If you are a member of the group with ID groupID then return true
     * @throws java.io.IOException
     * @throws org.xml.sax.SAXException
     * @throws com.aetrion.flickr.FlickrException
     */
    public boolean amIMemberOf(String groupID) throws IOException, SAXException, FlickrException
    {
        //Collection myGroups = flickr.getPoolsInterface().getGroups();

        Iterator it = myGroups.iterator();
        while (it.hasNext())
        {
            Group g = (Group) it.next();
            if(g.getId().equals(groupID))
                return true;
        }
        return false;
    }

    public void showContent(Map map) throws IOException, SAXException, FlickrException
    {

        Iterator allIter = map.keySet().iterator();

        while (allIter.hasNext()) {
            String setTitle = (String) allIter.next();

            Collection currentSet = (Collection) map.get(setTitle);
            System.out.print(setTitle + ":{ ");
            Iterator setIterator = currentSet.iterator();

            while (setIterator.hasNext()) {
                Photo p = (Photo) setIterator.next();
                System.out.print(p.getId() + " ");
            }
            System.out.println("}");
        }

    }
    /**
     *
     * @return Returns a map in that every key is a set of yours and the object is a photolist
     *  with the photos of this set with its info updated
     */
    private void allMyPhotosMap() throws IOException, SAXException, FlickrException
    {
        PhotosetsInterface pi = flickr.getPhotosetsInterface();
        PhotosInterface photoInt = flickr.getPhotosInterface();

        allPhotos = new HashMap();
        numberPhotos = 0;

        Iterator sets = pi.getList(this.nsid).getPhotosets().iterator();
        while (sets.hasNext())
        {
            Photoset set = (Photoset)sets.next();
            PhotoList photos;
            photos = pi.getPhotos(set.getId(), 500, 1);
            numberPhotos += photos.size();
            allPhotos.put(set.getTitle(), photos);
        }

        int notInSetPage = 1;
        PhotoList notInASet = new PhotoList();

        while (true)
        {
            PhotoList nis = photoInt.getNotInSet(100, notInSetPage);
            PhotoList nisUpdated;
            nisUpdated = nis;
            notInASet.addAll(nisUpdated);
            if (nis.size() < 100) break;
            notInSetPage++;
         }
         numberPhotos += notInASet.size();
         allPhotos.put("NotInASet", notInASet);
    }

    private void downloadPhotos()
    {
        Map newAllPhotos = new HashMap();

        JProgressBar pb = escotaView.getUpdatingData().getProgressBar();
        pb.setIndeterminate(false);
        pb.setMinimum(0);
        pb.setMaximum(numberPhotos);
        int i = 0;

        File directory = new File(System.getProperty("user.home") + File.separatorChar + ".escota" + File.separatorChar + "photos");

        if (!directory.exists()) directory.mkdir();
        
        Iterator allIter = allPhotos.keySet().iterator();

        while (allIter.hasNext()) {
            String setTitle = (String) allIter.next();

            Collection currentSet = (Collection) allPhotos.get(setTitle);
            Iterator setIterator = currentSet.iterator();
            File setDirectory = new File(directory, setTitle);

			setDirectory.mkdir();

            PhotoList newPhotos = new PhotoList();
            escotaView.getUpdatingData().setTitle("Downloading your photos");
            while (setIterator.hasNext()) {
                Photo p = (Photo) setIterator.next();
                //escotaView.getUpdatingData().getMessage().setText("Downloading: " + p.getTitle());

                try {
                    newPhotos.add(flickr.getPhotosInterface().getInfo(p.getId(), secret));

                    BufferedInputStream inStream = new BufferedInputStream(flickr.getPhotosInterface().getImageAsStream(p, Size.THUMB));
                    File newFile = new File(setDirectory, p.getId() + ".jpg");
                    FileOutputStream fos = new FileOutputStream(newFile);

                    int read;

                    while ((read = inStream.read()) != -1) {
                        fos.write(read);
                    }
                    fos.flush();
                    fos.close();
                    inStream.close();

                    String filePath = System.getProperty("user.home") + File.separatorChar + ".escota" + File.separatorChar + "photos" + File.separatorChar + setTitle + File.separatorChar + p.getId() + ".jpg";
                    ImageIcon icon = new ImageIcon(filePath);
                    escotaView.getUpdatingData().getMessage().setText(p.getTitle());
                    escotaView.getUpdatingData().getMessage().setIcon(icon);

                    pb.setValue(i);
                    i++;
                } catch (Exception ex) {
                    Log.error("Error downloading image from flickr", ex);
                    escotaView.showErrorMessage("Error downloading images from flickr", true);
                }
            }
            newAllPhotos.put(setTitle, newPhotos);
        }
        this.allPhotos = newAllPhotos;
    }

    /**
     *
     * @param photoList
     * @return Returns the same PhotoList but with the info of every photo updated
     */
    public PhotoList updateInfo(PhotoList photoList) throws IOException, SAXException, FlickrException
    {
        PhotoList res = new PhotoList();
        Iterator it = photoList.iterator();

        while(it.hasNext())
        {
            Photo photo = (Photo) it.next();

            Photo photoInfo = getPhotoInfo(photo);
            res.add(photoInfo);
        }
        return res;
    }
        /**
     *
     * @param photoList
     * @return Returns a PhotoList but with the info of every photoID updated
     */
    public PhotoList getPhotoInfoList(List photoIDList) throws IOException, SAXException, FlickrException
    {
        PhotoList res = new PhotoList();
        Iterator it = photoIDList.iterator();

        while(it.hasNext())
        {
            String photoID = (String) it.next();

            Photo photoInfo = getPhotoInfo(photoID);
            res.add(photoInfo);
        }
        return res;
    }

    public Photo getPhotoInfo(Photo photo) throws IOException, SAXException, FlickrException
    {
        return getPhotoInfo(photo.getId());
    }
    public Photo getPhotoInfo(String photoID) throws IOException, SAXException, FlickrException
    {
        Iterator allIter = allPhotos.keySet().iterator();

        while (allIter.hasNext()) {
            String setTitle = (String) allIter.next();
            Collection currentSet = (Collection) allPhotos.get(setTitle);
            Iterator setIterator = currentSet.iterator();
            while (setIterator.hasNext()) {
                Photo p = (Photo) setIterator.next();
                if(p.getId().equalsIgnoreCase(photoID))
                {
                    return p;
                }
            }
        }
        escotaView.showErrorMessage("Error finding a photo", true);
        return null;
    }



    /**
     *
     * @param photoList Show by console an information of every Photo of photoList
     */
    public void consoleOut(PhotoList photoList) throws IOException, SAXException, FlickrException
    {
        Iterator it = photoList.iterator();
        while(it.hasNext())
        {
            Photo p = (Photo) it.next();
            System.out.println("********************************");
            System.out.println("Title: " + p.getTitle());
            System.out.println("ID: " + p.getId());
            System.out.println("Comments: " + p.getComments());
            System.out.println("Views: " + p.getViews());
            System.out.println("Favs: " + numberFavorites(p));
            System.out.print("Tags: ");
            Iterator itTags = p.getTags().iterator();
            while(itTags.hasNext())
            {
                Tag tag = (Tag) itTags.next();
                if(itTags.hasNext())
                    System.out.print(tag.getValue() + ", ");
                else
                    System.out.println(tag.getValue());
            }
        }
    }
    private void retrieveMyGroups() throws IOException, SAXException, FlickrException
    {
            myGroups = flickr.getPoolsInterface().getGroups();
    }

    public void muestraMisGrupos()
    {
        try{
            Iterator it = flickr.getPoolsInterface().getGroups().iterator();

            while (it.hasNext())
            {
                Group g = (Group) it.next();
                System.out.println("Name: " + g.getName() + " ID: " + g.getId() );
            }
        }
        catch(Exception e)
        {
            System.out.println(e);
        }
    }

    public Boolean isAuthorized() {
        return authorized;
    }

    public Collection getMyGroups() {
        return myGroups;
    }
    public Map getAllPhotos()
    {
        return allPhotos;
    }

    public void setNsid(String nsid) {
        this.nsid = nsid;
    }
        /**
     *
     * @return Returns a PhotoList with all the photos in your Flickr account
     */
    public PhotoList allMyPhotosList() throws IOException, SAXException, FlickrException
    {
        Map map = this.allPhotos;
        PhotoList res = new PhotoList();

        Iterator allIter = map.keySet().iterator();

        while (allIter.hasNext()) {
            String setTitle = (String) allIter.next();

            Collection currentSet = (Collection) map.get(setTitle);
            Iterator setIterator = currentSet.iterator();

            while (setIterator.hasNext()) {
                Photo p = (Photo) setIterator.next();
                res.add(p);
            }
        }

        return res;
    }
}