/*
    This file is part of Escota.

    Escota is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Escota is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Escota.  If not, see <http://www.gnu.org/licenses/>.
 */

package escota.flickr;

import com.aetrion.flickr.photos.PhotoList;
import escota.EscotaTaskGenerator;
import escota.EscotaView;
import escota.utils.Log;


/**
 *
 * @author AGuS
 */
public class GenerateTasksThread extends Thread {
    private String selectedSet;
    private int[] selectedIndices;
    private Conexion conexion;
    private Rule rule;
    private TaskQueue taskQueue;
    private EscotaView ev;
    public GenerateTasksThread(String selectedSet, int[] selectedIndices, Conexion conexion, Rule rule, TaskQueue taskQueue, EscotaView ev)
    {
        super();
        this.selectedIndices = selectedIndices;
        this.selectedSet = selectedSet;
        this.conexion = conexion;
        this.rule = rule;
        this.taskQueue = taskQueue;
        this.ev = ev;

    }

    @Override
    public void run() {
        if(selectedSet!=null)
        {
            generateSelected();
        }
        else
        {
            generateAll();
        }
    }
    private void generateAll()
    {
        EscotaTaskGenerator tg = ev.getTaskGenerator();
        try {
                PhotoList photos = conexion.allMyPhotosList();
                
                while (tg == null) {
                sleep(1000);
                tg = ev.getTaskGenerator();
                }

                TaskQueue generated = rule.generateTasks(photos, conexion,tg);
                Log.debug(generated.size() + " tasks generated");
                taskQueue.addAll(generated);

            }
        catch (Exception ex) {
            Log.error("Error generating tasks", ex);
            ev.showErrorMessage("Error generating tasks", false);
        }
        Log.debug("Cerrando Task Generator");
        tg.close();
        Log.debug("Cerrando Photo Chooser");
        ev.getPhotoChooser().close();
    }

    private void generateSelected()
    {
        PhotoList photos = new PhotoList();
        final PhotoList photoList = (PhotoList) conexion.getAllPhotos().get(selectedSet);
        EscotaTaskGenerator tg = ev.getTaskGenerator();
        try {


            while (tg == null) {
                sleep(1000);
                tg = ev.getTaskGenerator();
            }

            //rule.generateTasks(photos, conexion, tg);
            if (selectedIndices.length > 0) {
                for (int i = 0; i < selectedIndices.length; i++) {
                    photos.add(photoList.get(i));
                }
                TaskQueue generated = rule.generateTasks(photos, conexion,tg);
                Log.debug(generated.size() + " tasks generated");
                taskQueue.addAll(generated);

            }
        } catch (Exception ex) {
            Log.error("Error generating tasks", ex);
            ev.showErrorMessage("Error generating tasks", false);
        }
        Log.debug("Cerrando Task Generator");
        tg.close();
        Log.debug("Cerrando Photo Chooser");
        ev.getPhotoChooser().close();
    }
}
