/*
    This file is part of Escota.

    Escota is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Escota is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Escota.  If not, see <http://www.gnu.org/licenses/>.
 */

package escota.flickr;

import com.aetrion.flickr.FlickrException;
import com.aetrion.flickr.photos.Photo;
import java.io.IOException;
import java.io.Serializable;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import org.xml.sax.SAXException;

/**
 *
 * @author AGuS
 */
public class Task implements Serializable {
    private Date nextTry;
    private Rule rule;

    private String photoID;
    private String photoTitle;
    private String groupID;
    private String groupName;
    private boolean isAddition;

    public Task(String photoID, String groupID, Rule rule, Date nextTry, boolean addition)
    {
        this.photoID = photoID;
        this.groupID = groupID;
        this.rule = rule;
        this.nextTry = nextTry;
        this.isAddition = addition;
    }

    public Task(String photoID, String photoTitle, String groupID, String groupName, Rule rule, boolean addition)
    {
        this.photoID = photoID;
        this.photoTitle = photoTitle;
        this.groupID = groupID;
        this.groupName = groupName;
        this.rule = rule;
        this.nextTry = new Date();
        this.isAddition = addition;
    }

    public String getGroupID() {
        return groupID;
    }

    public String getPhotoID() {
        return photoID;
    }

    public boolean canDoIt()
    {
        Date now = new Date();
        boolean canDoIt = nextTry.before(now);
        return canDoIt;
    }
    public boolean satisfiedRule(Conexion conexion) throws IOException, SAXException, FlickrException
    {
        Photo photo = conexion.getPhotoInfo(photoID);
        return rule.checkRule(photo, conexion);
    }

    public void addDays(int days)
    {
        Calendar cal = new GregorianCalendar();
        cal.add(Calendar.DATE, days);
        nextTry = cal.getTime();
    }

    public Rule getRule() {
        return rule;
    }

    public String getGroupName() {
        return groupName;
    }

    public String getPhotoTitle() {
        return photoTitle;
    }

    public Date getNextTry()
    {
        return this.nextTry;
    }

    public boolean isIsAddition() {
        return isAddition;
    }

    public void setIsAddition(boolean isAddition) {
        this.isAddition = isAddition;
    }
    

    @Override
    public String toString()
    {
        return "Task: group = " + groupID + " nextTry=" + nextTry + " photo = " + photoID;
    }
}
